import json
from flask import Flask, jsonify
app = Flask(__name__)

@app.route('/products/laptops')
def get_products():
    with open('out/out.json') as f:
        data = json.load(f)
    return jsonify(data)

@app.route('/')
def hi():
    return {"message": "Welcome to API Products"}


import json
import requests
from bs4 import BeautifulSoup
from decimal import Decimal
from re import sub
from utils import price_to_float

lst_result = [] 

class Scraper:
    
    def __init__(self, category):
        self.category = category
        with open('./ecommerces.json') as f:
            data = json.load(f)
        self.all_resources = data

    def ln(self): 
        req = requests.get(self.all_resources[self.category]["linio"])
        soup = BeautifulSoup(req.text, "lxml")
        lst_result = []
        for item in soup.find_all("div",{"class": "catalogue-product"}):
            title = item.find("a").get('title')
            link  = item.find("a").get('href')
            price = item.find("a").find("div",{'class':'price-section'}).find("meta",{"itemprop":"price"}).get("content")
            link_trans = 'https://linio.com.pe'+link
            price_formated = price_to_float(price)
            lst_result.append({"title":title, "price": price_formated, "link": link_trans})
        return lst_result
    def fb(self):
        req = requests.get(self.all_resources[self.category]["falabella"])
        soup = BeautifulSoup(req.text, "lxml")
        lst_result = []
        ##TODO this need to refactor  
        if self.category == 'laptop':
            items = soup.find("div",{"id": "testId-searchResults-products"}).find_all("div",{"class": "search-results-list"})
        else:
            items = soup.find("div",{"id": "testId-searchResults-products"}).find_all("div",{"class": "search-results-4-grid"})
        for item in items:
            link  = item.find("a").get('href')
            title = item.find("b",{"class": "pod-subTitle"}).text
            price = item.find("ol",{"class":"fa--prices"}).find("li",{"class": "price-0"}).get("data-undefined-price")
            price_formated = price_to_float(price)
            lst_result.append({"title":title, "price": price_formated, "link": link})
        return lst_result

    def rp(self):
        req = requests.get(self.all_resources[self.category]["ripley"])
        soup = BeautifulSoup(req.text, "lxml")

        lst_result = []
        if self.category == 'laptop':
            items = soup.find("div",{"id": "catalog-page"}).find_all("div",{"class": "ProductItem"})
            for item in items:
                title = item.find("a",{"class":"ProductItem__Name"}).text
                link  = item.find("a",{"class":"ProductItem__Name"}).get("href")
                price_el = item.find("li",{"class": "catalog-prices__offer-price"})
                price2_el  = item.find("li",{"class": "catalog-prices__list-price"})
                price = price_el.text if price_el != None else None
                if price == '':
                    price = price2_el.text if price2_el != None else None
                
                price_formated = price_to_float(price)
                link_trans = 'https://simple.ripley.com.pe'+link
                lst_result.append({"title":title, "price": price_formated, "link": link_trans})
        else:
            items = soup.find("div",{"class": "catalog-container"}).find_all("a",{"class": "catalog-product-item"})
            for item in items:
                title = item.find("div",{"class": "catalog-product-details__name"}).text
                link =  item.get("href")
                price_el = item.find("li",{"class": "catalog-prices__offer-price"})
                price2_el  = item.find("li",{"class": "catalog-prices__list-price"})
                price = price_el.text if price_el != None else None
                if price == '':
                    price = price2_el.text if price2_el != None else None
                price_formated = price_to_float(price)
                link_trans = 'https://simple.ripley.com.pe'+link
                lst_result.append({"title":title, "price": price_formated, "link": link_trans})
        return lst_result

    def oc(self):
        req = requests.get(self.all_resources[self.category]["oechsle"])
        soup = BeautifulSoup(req.text, "lxml")
        lst_result = []    
        items = soup.find("div", {"class":"main"}).find_all("div",{"class": "product"})
        for item in items:
            title = item.get("data-name")
            link = item.get("data-link")
            price = item.find("span", {"class": "BestPrice"}).text
            price_formated = price_to_float(price)
            lst_result.append({"title":title, "price": price_formated, "link": link})
        return lst_result
    
    def all_ecommerce(self):
        return self.ln() + self.fb() + self.rp() + self.oc()




import argparse
from scrap import Scraper
from utils import export_to

ap = argparse.ArgumentParser(prog='cli-export',
                              usage='%(prog)s [options] path',)

ap.add_argument("-c","--category", required=True, 
   choices=["laptop","celular","audifono","monitor"], default="laptop", help="category to export")

ap.add_argument("-o", "--output", required=False,
   help="file to export data", default="out")
ap.add_argument("-f", "--format", required=False,
   help="format to export data", choices=["csv","json"], default="csv")

args = vars(ap.parse_args())

scrap = Scraper(args['category'])
all_products = scrap.all_ecommerce()
exp = export_to(all_products, filename=args['output'], format=args['format'])



import csv
import re
import os
import json
def price_to_float(x):
    rs = 0.0
    if x != None:
        rs = re.search("-?\d+(?:,\d{3})*(?:\.\d+)?", x).group()
        rs = float(rs.replace(",",""))
    return rs

def export_to(data, format="csv", filename="out"):
    all_formats = ("csv","json")
    filename_path = os.getcwd() + "/out/"+filename+"."+format
    #FIXME go to class in fhe future
    if not os.path.exists(os.path.dirname(filename_path)):
        try:
            os.makedirs(os.path.dirname(filename_path))
        except OSError as exc: # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise

    if format in all_formats:
        if format == "csv":
            keys = data[0].keys()
            with open(filename_path, "w", newline="") as f:
                dict_writer = csv.DictWriter(f, keys)
                dict_writer.writeheader()
                dict_writer.writerows(data)
        if format == "json":
            with open(filename_path, 'w') as outfile:
                json.dump(data, outfile)

        print("Exported: %s items" % str(len(data)))



import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='ecope',
    version='0.1.3',
    scripts=['./scripts/ecope'],
    author='Diego Machaca',
    author_email = 'diegomachaca@gmail.com',   
    url = 'https://github.com/djego/ecommerce-peru-scrap-cli',
    keywords = ['scraping', 'peru', 'ecommerce','csv','json'],
    description='Ecommerce Perú Scrap CLI is a project open source that extract products data by category and export to csv, json and other structure format files',
    packages=['application'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        'setuptools',
        'beautifulsoup4 == 4.9.1',
        'html5lib == 1.1',
        'lxml == 4.5.2',
        'six == 1.15.0',
        'soupsieve == 2.0.1',
        'webencodings == 0.5.1',
        'requests == 2.24.0'
    ],
    python_requires='>=3.7'
)